------------------
--- suportadas na nova versao do terrahs

module TerraHS.Algebras.Base.Tree where
-- mdulo para definio operaes e do formato geral da rvore
-- para um tipo t qualquer

-- definio das operaes
class SimpleTree t where
	leaf                :: a -> t a
	tnull               :: t a
	branch              :: a -> t a -> t a -> t a
	getData             :: t a -> a
	setData             :: t a -> a -> t a
	left, right         :: t a -> t a
	allLeaves 			:: t a -> [t a]
	allBranches 			:: t a -> [t a]
	allObjects			:: t a -> [t a]
	isLeaf              :: t a -> Bool
	
-- definio de uma estrutura de rvore de um tipo a qualquer
-- a rvore pode ser uma folha tipo a, um n interno tip a com duas sub-rvores tipo a ou um valor nulo
-- Tree: type constructor
-- Leaf and Branch: data constructors
data Tree a  = Leaf a | Branch  a (Tree a) (Tree a) | NULL  deriving (Show, Eq)

-- definio dos axiomas (implementao das operaes com suas restries)
instance SimpleTree Tree  where	
	leaf                  		= Leaf				-- retorna uma folha tipo t com o objeto a
	tnull						= NULL				-- retorna nulo para ser colocado na rvore
	branch   					= Branch			-- retorna um n interno 3 componentes tipo t para o objeto a
	getData  (Leaf a)       	= a					-- recupera o objeto na folha da rvore
	getData  (Branch a l r) 	= a					-- recupera o objeto no n interno da rvore
	setData  (Leaf _) a         = (Leaf a)			-- define um objeto a de entrada para uma folha
	setData  (Branch _ l r) a   = (Branch a l r)	-- define um objeto tipo a para um n da rvore com as subrvores da direita e da esquerda
	left  (Branch a l r)    	= l					-- recupera a subrvore da esquerda
	right (Branch a l r)    	= r					-- recupera a subrvore da direita
	isLeaf   (Leaf _)       	= True				-- testa se um objeto  uma folha
	isLeaf   _              	= False			
	allBranches tr								-- recupera todos os ns internos sem as folhas
		| not (isLeaf tr ) = tr : (allBranches (left tr))  ++ (allBranches (right tr))  
		| (isLeaf tr) = []
		
	allLeaves tr									-- recupera todas as folhas que compe a rvore 
		| not (isLeaf tr ) = (allLeaves (left tr)) ++ (allLeaves (right tr))  
		| (isLeaf tr) = [tr]
		
	allObjects tr									-- recupera todas as folhas que compe a rvore 
		| not (isLeaf tr ) = tr : (allObjects (left tr))  ++ (allObjects (right tr))  
		| (isLeaf tr) = [tr]
		

fringe :: Tree a -> [a]   -- A different definition of fringe
fringe (Leaf x) = [x]
fringe (Branch o x y) = fringe x